
##rm(list=ls(all=TRUE))

##library(MASS)


##Soft thresholding
##lambda can be a matrix with same dim as C
softTH <- function(C,Lam) {
   Ct <- abs(C)-Lam
   Ct[Ct<0] <- 0
   sign(C)*Ct
} 

#########Some correlation structure
CorrAR <- function(p,rho){  
  Sigma <- matrix(nrow=p,ncol=p,NA)
  for(i in 1:p){
    for(j in 1:p){
      Sigma[i,j] <- rho^(abs(i-j))
    }
  }
  Sigma
}

CorrCS <- function(p,rho){  
  Sigma <- matrix(nrow=p,ncol=p,rho)
  diag(Sigma) <- 1
  Sigma
}


###########a simulation model#########
RRR.sim1 <- function(n=100,p=25,q=25,snr=0.1,rstar=10,rho.x=0.5,rho.e=0,SigmaX=CorrAR,SigmaE=CorrAR){
   
  require(MASS)
  ##simulate data
  C0 <- matrix(ncol=rstar,nrow=p,rnorm(rstar*p))
  C1 <- matrix(ncol=rstar,nrow=q,rnorm(rstar*q))
  C <- snr*C0%*%t(C1)

  X <- mvrnorm(n, rep(0,p), SigmaX(p,rho.x)) 
  E <- mvrnorm(n, rep(0,q), SigmaE(q,rho.e)) 

  Y<-X%*%C+E
  
  list(Y=Y,X=X,C=C)  

}


RRR.sim2 <- function(n=20,p=100,q=25,snr=0.1,rstar=5,rx=10,rho.x=0.5,rho.e=0,SigmaX=CorrAR,SigmaE=CorrAR){
   
  require(MASS)
  ##simulate data
  C0 <- matrix(ncol=rstar,nrow=p,rnorm(rstar*p))
  C1 <- matrix(ncol=rstar,nrow=q,rnorm(rstar*q))
  C <- snr*C0%*%t(C1)
  
  a.eig <- eigen(SigmaX(p,rho.x))
  SigmaX.sqrt <- a.eig$vectors %*% diag(sqrt(a.eig$values)) %*% solve(a.eig$vectors)
  X0 <- matrix(ncol=rx,nrow=n,rnorm(rx*n))
  X1 <- matrix(ncol=rx,nrow=p,rnorm(rx*p))
  X <- X0%*%t(X1)%*%SigmaX.sqrt
  
  E <- mvrnorm(n, rep(0,q), SigmaE(q,rho.e)) 

  Y<-X%*%C+E
  
  list(Y=Y,X=X,C=C)  

}




RRR <- function(Y,X,nrank=1,weight=FALSE,Gamma=diag(ncol(Y)),ypy.svd=TRUE,c.svd=FALSE){

   q <- ncol(Y)
   n <- nrow(Y)
   p <- ncol(X)
   S_yx <- t(Y)%*%X
   S_xx <- t(X)%*%X
   
   ##S_xx_inv <- tryCatch(solve(S_xx+0.1*diag(p)),error=function(e)ginv(S_xx))
   S_xx_inv <- tryCatch(ginv(S_xx),error=function(e)solve(S_xx+0.1*diag(p)))
   ##S_xx_inv <- ginv(S_xx)
   if(sum(is.na(S_xx_inv))>0){
        S_xx_inv <- solve(S_xx+0.1*diag(p))
   }
   C_ls <- S_xx_inv%*%t(S_yx)
   
   
   ###If use a weight matrix
   if(weight==TRUE){
      eigenGm <- eigen(Gamma)
      sqrtGm <- eigenGm$vectors%*%diag(sqrt(eigenGm$values))%*%t(eigenGm$vectors)
      sqrtinvGm <- eigenGm$vectors%*%diag(sqrt(eigenGm$values^(-1)))%*%t(eigenGm$vectors)       
      
      if(ypy.svd){
        XC <- X%*%C_ls%*%sqrtGm
        ##SVD may not converge
        svdXC <- tryCatch(svd(XC,nu=nrank,nv=nrank),error=function(e)2)
        if(tryCatch(svdXC==2,error=function(e)3)==3){
            A <- svdXC$v[,1:nrank]
            Ad <- (svdXC$d[1:nrank])^2
        }else{
            ypy.svd <- FALSE
        }
      }
      if(!ypy.svd){
        SS <- sqrtGm%*%S_yx%*%C_ls%*%sqrtGm
        SS <- (SS+t(SS))/2
        eigenSS <- eigen(SS,symmetric=TRUE)
        A <- as.matrix(eigenSS$vectors[,1:nrank])
        Ad <- eigenSS$values[1:nrank]   
      }
      
      AA <- A%*%t(A)
      C_rr <-C_ls%*%sqrtGm%*%AA%*%sqrtinvGm
      
   }else{
     
     if(ypy.svd){
        XC <- X%*%C_ls
        ##svdXC <- svd(XC,nu=nrank,nv=nrank)
        ##SVD may not converge
        svdXC <- tryCatch(svd(XC,nu=nrank,nv=nrank),error=function(e)2)
        if(tryCatch(svdXC==2,error=function(e)3)==3){
            A <- svdXC$v[,1:nrank]
            Ad <- (svdXC$d[1:nrank])^2
        }else{
            ypy.svd <- FALSE
        }
     }
     if(!ypy.svd){
        SS <- S_yx%*%C_ls
        SS <- (SS+t(SS))/2
        eigenSS <- eigen(SS,symmetric=TRUE)
        A <- as.matrix(eigenSS$vectors[,1:nrank])
        Ad <- eigenSS$values[1:nrank]   
     }
      
     AA <- A%*%t(A)
     C_rr <-C_ls%*%AA
   }
   
   if(c.svd){
     svd_C <- svd(C_rr,nv=nrank,nu=nrank)
     U <- as.matrix(svd_C$u[,1:nrank])
     V <- as.matrix(svd_C$v[,1:nrank])
     D <- diag(svd_C$d[1:nrank],nrow=nrank)
     ##return ls estimator C_ls, reduced-rank estimator C_rr
     ##return SVD of C_rr
     list(A=A,Ad=Ad,C_ls=C_ls,C_rr=C_rr,U=U,V=V,D=D,C=C_rr,rank=nrank)
   }else{
        list(A=A,Ad=Ad,C_ls=C_ls,C_rr=C_rr,C=C_rr,rank=nrank)        
   }   
}





##Reduced-rank Ridge Regression
RRRR <- function(Y,X,nrank=min(dim(Y),dim(X)),lambda=1,ypy.svd=TRUE,c.svd=FALSE){

   q <- ncol(Y)
   n <- nrow(Y)
   p <- ncol(X)
   S_yx <- t(Y)%*%X
   ##This is a key difference
   S_xx <- t(X)%*%X + lambda*diag(p)

   ##S_xx_inv <- tryCatch(solve(S_xx+0.1*diag(p)),error=function(e)ginv(S_xx))
   ##S_xx_inv <- ginv(S_xx)
   ##Use the Woodbury matrix identity
   if(lambda!=0){
     S_xx_inv <- 1/lambda*diag(p) - lambda^(-2)*t(X)%*%ginv(diag(n)+lambda^(-1)*X%*%t(X))%*%X
   }else{
     ##S_xx_inv <- ginv(S_xx)
     S_xx_inv <- tryCatch(solve(S_xx+0.1*diag(p)),error=function(e)ginv(S_xx))
     if(sum(is.na(S_xx_inv))>0){
        S_xx_inv <- solve(S_xx+0.1*diag(p))
     }
   }
   
   C_ls <- S_xx_inv%*%t(S_yx)

   if(ypy.svd){
     ##This is another key difference
     XC <- rbind(X,sqrt(lambda)*diag(p))%*%C_ls
     svdXC <- tryCatch(svd(XC,nu=nrank,nv=nrank),error=function(e)2)
     if(tryCatch(svdXC==2,error=function(e)3)==3){
        A <- svdXC$v[,1:nrank]
        Ad <- (svdXC$d[1:nrank])^2
     }else{
       ypy.svd <- FALSE
     }
   }  
   if(!ypy.svd){
     SS <- S_yx%*%C_ls
     SS <- (SS+t(SS))/2
     eigenSS <- eigen(SS,symmetric=TRUE)
     A <- as.matrix(eigenSS$vectors[,1:nrank])
     Ad <- eigenSS$values[1:nrank]   
   }
      
   AA <- A%*%t(A)
   C_rr <-C_ls%*%AA
   
   if(c.svd){
     svd_C <- svd(C_rr,nv=nrank,nu=nrank)
     U <- as.matrix(svd_C$u[,1:nrank])
     V <- as.matrix(svd_C$v[,1:nrank])
     D <- diag(svd_C$d[1:nrank],nrow=nrank)

     ##return ls estimator C_ls, reduced-rank estimator C_rr
     ##return SVD of C_rr
     list(A=A,Ad=Ad,C_ls=C_ls,C_rr=C_rr,U=U,V=V,D=D,C=C_rr,rank=nrank)
   }else{
     list(A=A,Ad=Ad,C_ls=C_ls,C_rr=C_rr,C=C_rr,rank=nrank)        
   }
}





###Determine rank using Cross Validation
####Note that the rank goes from 0 to rankmax
RRR.cv <- function(Y,X,nfold=10,rankmax=min(dim(Y),dim(X)),norder=NULL,fold.drop=0,ypy.svd=TRUE,c.svd=FALSE){

  p <- ncol(X)
  q <- ncol(Y)
  n <- nrow(Y)

  ndel <- round(n/nfold)
  if(is.null(norder)){
     norder <- sample(1:n,n)
  }

  cr_path <- matrix(ncol=nfold,nrow=rankmax+1,NA)

  for(f in 1:nfold){
    ##determine cr sample
    if(f!=nfold){
        iddel <- norder[(1+ndel*(f-1)):(ndel*f)]
    }else{
        iddel <- norder[(1+ndel*(f-1)):n]
    }
    ndel <- length(iddel)
    nf <- n - ndel
    idkeep <- (1:n)[-iddel]

    Xf <- X[-iddel,]
    Xfdel <- X[iddel,]
    Yf <- Y[-iddel,]
    Yfdel <- Y[iddel,]

    ini <- RRR(Yf,Xf,nrank=rankmax,ypy.svd=ypy.svd,c.svd=c.svd)
    C_ls <- ini$C_ls
    A <- ini$A
    
#    temp <- array(dim=c(q,q,rankmax))
#    for(i in 1:rankmax)temp[,,i] <- A[,i]%*%t(A[,i])
#    tempfit <- Xfdel%*%C_ls
#    tempA <- 0
#    while(i <= rankmax){
#      tempA <- tempA + temp[,,i]
#      ##C <- C_ls%*%A[,1:i]%*%t(A[,1:i])
#      ##cr_path[i,f] <- sum((Yfdel-Xfdel%*%C)^2) 
#      cr_path[i,f] <- sum((Yfdel-tempfit%*%tempA)^2)
#      i=i+1
#    }
    
    tempFit <- Xfdel%*%C_ls    
    tempC <- matrix(nrow=q,ncol=q,0)
    ##system.time(
    for(i in 1:rankmax){
      tempC <- tempC + A[,i]%*%t(A[,i])
      cr_path[i+1,f] <- sum((Yfdel-tempFit%*%tempC)^2)         
    }
    ##)
    ##The null solution
    cr_path[1,f] <- sum(Yfdel^2)

  }

  ##report total CVE
  ##if nfold > 5, it is allowed to drop #fold.drop folds with largest total error.
  ##Emperically, this shows robustness.
  index <- order(apply(cr_path,2,sum))[1:ifelse(nfold>5,nfold-fold.drop,nfold)]
  crerr <- apply(cr_path[,index],1,sum)/length(index)*nfold
  minid <- which.min(crerr)
  ###estimated rank
  rankest <- minid - 1

  ##Refit the model with all data 
  ini <- RRR(Y,X,nrank=rankmax)
  C_ls <- ini$C_ls
  A <- ini$A

  if(rankest==0){
    list(cr_path=cr_path, CRE=crerr,norder=norder,C=matrix(nrow=p,ncol=q,0),rank=0,C_ls=C_ls)
  } else {
    A <- ini$A
    list(cr_path=cr_path, CRE=crerr,norder=norder,C_ls=C_ls, C=C_ls%*%A[,1:rankest]%*%t(A[,1:rankest]),rank=rankest)
  }
}









RRR.ideal <- function(Y,X,Ytest,Xtest,rankmax=min(dim(Y),dim(X))){

  p <- ncol(X)
  q <- ncol(Y)
  n <- nrow(Y)

  cr_path <- vector()

  ini <- RRR(Y,X,nrank=rankmax)
  C_ls <- ini$C_ls
  A <- ini$A

  tempFit <- Xtest%*%C_ls
  for(i in 1:rankmax){
     tempC <- A[,1:i]%*%t(A[,1:i])
     cr_path[i+1] <- sum((Ytest-tempFit%*%tempC)^2)         
  }
  
#  for(i in 1:rankmax){
#     C <- C_ls%*%A[,1:i]%*%t(A[,1:i])
#     cr_path[i] <- sum((Ytest-Xtest%*%C)^2)
#  }
  
  
  cr_path[1] <- sum(Ytest^2)
   
  minid <- which.min(cr_path)
  rankest <- minid - 1

  if(rankest==0){
    list(CRE=cr_path,C_ls=C_ls,C=matrix(nrow=p,ncol=q,0),rank=0)
  } else {
    list(CRE=cr_path,C_ls=C_ls,C=C_ls%*%A[,1:rankest]%*%t(A[,1:rankest]),rank=rankest)
  }

}



###Determine rank using Cross Validation
####Note that the rank goes from 0 to rankmax
####Robustified RRR--------------------------------------------------#########
RRRR.cv <- function(Y,X,nfold=10,rankmax=min(dim(Y),dim(X)),
                    nlam=100,lambda=seq(0,100,length=nlam),norder=NULL,nest.tune=FALSE,fold.drop=0){

  p <- ncol(X)
  q <- ncol(Y)
  n <- nrow(Y)

  ndel <- round(n/nfold)
  if(is.null(norder)){
     norder <- sample(1:n,n)
  }

  cr_path <- array(dim=c(nlam,rankmax+1,nfold),NA)

  for(f in 1:nfold){
    ##determine cr sample
    if(f!=nfold){
        iddel <- norder[(1+ndel*(f-1)):(ndel*f)]
    }else{
        iddel <- norder[(1+ndel*(f-1)):n]
    }
    ndel <- length(iddel)
    nf <- n - ndel
    ##idkeep <- (1:n)[-iddel]

    Xf <- X[-iddel,]
    Xfdel <- X[iddel,]
    Yf <- Y[-iddel,]
    Yfdel <- Y[iddel,]
      
    for(ll in 1:nlam){
    
      ##Yflam <- rbind(Yf,matrix(nrow=p,ncol=q,0))
      ##Xflam <- rbind(Xf,sqrt(lam[ll])*diag(p))  
      ##system.time( 
      ##ini <- RRR(Yflam,Xflam,nrank=rankmax)
      ##)
      ##system.time(
      ##)

      ###Note that C_ls is the ridge estimator
      ###C_ls and A are different for each lambda
      ini <- RRRR(Yf,Xf,lambda[ll],nrank=rankmax)      
      C_ls <- ini$C_ls
      A <- ini$A

      tempFit <- Xfdel%*%C_ls
      ##for(i in 1:rankmax){
      ##  tempC <- A[,1:i]%*%t(A[,1:i])
      ##  cr_path[ll,i,f] <- sum((Yfdel-tempFit%*%tempC)^2)         
      ##}
      tempC <- matrix(nrow=nrow(A),ncol=nrow(A),0)
      ##system.time(
      for(i in 1:rankmax){
        tempC <- tempC + A[,i]%*%t(A[,i])
        cr_path[ll,i+1,f] <- sum((Yfdel-tempFit%*%tempC)^2)         
      }
      ##)
      ##The null solution      
      cr_path[ll,1,f] <- sum(Yfdel^2)
    }
    
  }

  index <- order(apply(cr_path,3,sum))[1:ifelse(nfold>5,nfold-fold.drop,nfold)]    
  crerr <- apply(cr_path[,,index],c(1,2),sum)/length(index)*nfold
  minid <- which.min(crerr)
  minid2 <- ceiling(minid/nlam)
  rankest <- minid2 - 1
  minid1 <- minid - nlam*(minid2-1)

  ###If nest.tune, then fine tune lambda again#####
 if(nest.tune==TRUE){
  ####update lambda####
  ##lam <- seq(lam[max(minid1-1,1)],lam[min(minid1+1,nlam)],length=nlam)
  lambda <- exp(seq(log(lambda[max(minid1-1,1)]+1e-4),log(lambda[min(minid1+1,nlam)]),length=nlam))

  cr_path <- array(dim=c(nlam,rankmax+1,nfold),NA)
  ndel <- round(n/nfold)

  for(f in 1:nfold){
    ##determine cr sample
    if(f!=nfold){
        iddel <- norder[(1+ndel*(f-1)):(ndel*f)]
    }else{
        iddel <- norder[(1+ndel*(f-1)):n]
        ndel <- length(iddel)
    }
    nf <- n - ndel
    ##idkeep <- (1:n)[-iddel]

    Xf <- X[-iddel,]
    Xfdel <- X[iddel,]
    Yf <- Y[-iddel,]
    Yfdel <- Y[iddel,]
      
    for(ll in 1:nlam){
    
      ##Yflam <- rbind(Yf,matrix(nrow=p,ncol=q,0))
      ##Xflam <- rbind(Xf,sqrt(lam[ll])*diag(p))  
      ##ini <- RRR(Yflam,Xflam,nrank=rankmax)

      ini <- RRRR(Yf,Xf,lambda[ll],nrank=rankmax)      
      C_ls <- ini$C_ls
      A <- ini$A

      tempFit <- Xfdel%*%C_ls
      ##for(i in 1:rankmax){
      ##  tempC <- A[,1:i]%*%t(A[,1:i])
      ##  cr_path[ll,i,f] <- sum((Yfdel-tempFit%*%tempC)^2)         
      ##}
      tempC <- matrix(nrow=nrow(A),ncol=nrow(A),0)
      ##system.time(
      for(i in 1:rankmax){
        tempC <- tempC + A[,i]%*%t(A[,i])
        cr_path[ll,i+1,f] <- sum((Yfdel-tempFit%*%tempC)^2)         
      }
      ##)           
      
#     for(i in 1:rankmax){
#       C <- C_ls%*%A[,1:i]%*%t(A[,1:i])
#       cr_path[ll,i,f] <- sum((Yfdel-Xfdel%*%C)^2)
#     }

      ###The null solution 
      cr_path[ll,1,f] <- sum(Yfdel^2)
    }
    
  }

  index <- order(apply(cr_path,3,sum))[1:ifelse(nfold>5,nfold-fold.drop,nfold)]    
  crerr <- apply(cr_path[,,index],c(1,2),sum)/length(index)*nfold
  minid <- which.min(crerr)
  minid2 <- ceiling(minid/nlam)
  rankest <- minid2 - 1
  minid1 <- minid - nlam*(minid2-1)
 } 
   


  if(rankest==0){
    list(cr_path=cr_path,CRE=crerr,norder=norder,C=matrix(nrow=p,ncol=q,0),rank=0)
  } else {
    
    fit <- RRRR(Y,X,lambda[minid1],nrank=rankmax)      
    C_lslam <- fit$C_ls
    A <- fit$A
    list(cr_path=cr_path,CRE=crerr,ID=c(minid1,minid2),norder=norder,C=C_lslam%*%A[,1:rankest]%*%t(A[,1:rankest]),rank=rankest,lam=lambda[minid1])
  }
  
}






####Ideally tuned Robustified RRR-----------------------------------------#########
RRRR.ideal <- function(Y,X,Ytest,Xtest,rankmax=min(dim(Y),dim(X)),
                    nlam=100,lambda=seq(0,100,length=nlam),nest.tune=FALSE){
                    
  p <- ncol(X)
  q <- ncol(Y)
  n <- nrow(Y)

  cr_path <- matrix(nrow=nlam,ncol=rankmax+1,NA)

  for(ll in 1:nlam){
    
    if(lambda[ll]==0){
      Ylam <- Y
      Xlam <- X
    }else{
      Ylam <- rbind(Y,matrix(nrow=p,ncol=q,0))
      Xlam <- rbind(X,sqrt(lambda[ll])*diag(p))  
    }
    
    ini <- RRR(Ylam,Xlam,nrank=rankmax)
    C_ls <- ini$C_ls
    A <- ini$A
    tempFit <- Xtest%*%C_ls
    for(i in 1:rankmax){
      tempC <- A[,1:i]%*%t(A[,1:i])
      cr_path[ll,i+1] <- sum((Ytest-tempFit%*%tempC)^2)         
    }
#    for(i in 1:rankmax){
#      C <- C_ls%*%A[,1:i]%*%t(A[,1:i])
#      cr_path[ll,i] <- sum((Ytest-Xtest%*%C)^2)
#    }
      cr_path[ll,1] <- sum(Ytest^2)
  }

  minid <- which.min(cr_path)
  minid2 <- ceiling(minid/nlam)
  rankest <- minid2 - 1 
  minid1 <- minid - nlam*(minid2-1)


  ##update lambda
 if(nest.tune==TRUE){
  lambda <- seq(lambda[max(minid1-1,1)],lambda[min(minid1+1,nlam)],length=nlam)  
  cr_path <- matrix(nrow=nlam,ncol=rankmax+1,NA)

  for(ll in 1:nlam){
    
    if(lambda[ll]==0){
      Ylam <- Y
      Xlam <- X
    }else{
      Ylam <- rbind(Y,matrix(nrow=p,ncol=q,0))
      Xlam <- rbind(X,sqrt(lambda[ll])*diag(p))  
    }
    
    ini <- RRR(Ylam,Xlam,nrank=rankmax)
    C_ls <- ini$C_ls
    A <- ini$A

    tempFit <- Xtest%*%C_ls
    for(i in 1:rankmax){
      tempC <- A[,1:i]%*%t(A[,1:i])
      cr_path[ll,i+1] <- sum((Ytest-tempFit%*%tempC)^2)         
    }
#    for(i in 1:rankmax){
#      C <- C_ls%*%A[,1:i]%*%t(A[,1:i])
#      cr_path[ll,i] <- sum((Ytest-Xtest%*%C)^2)
#    }
      cr_path[ll,1] <- sum(Ytest^2)
  }

  minid <- which.min(cr_path)
  minid2 <- ceiling(minid/nlam)
  rankest <- minid2 - 1 
  minid1 <- minid - nlam*(minid2-1)
 }


  if(rankest==0){
    list(CRE=cr_path,C=matrix(nrow=p,ncol=q,0),rank=0)
  } else {
    
    if(lambda[minid1]==0){
      Ylam <- Y
      Xlam <- X
    }else{
      Ylam <- rbind(Y,matrix(nrow=p,ncol=q,0))
      Xlam <- rbind(X,sqrt(lambda[minid1])*diag(p))  
    }
    fit <- RRR(Ylam,Xlam,nrank=rankmax)
    C_lslam <- fit$C_ls
    A <- fit$A
    C <- C_lslam%*%A[,1:rankest]%*%t(A[,1:rankest])
  
    list(CRE=cr_path,C=C,rank=rankest,lam=lambda[minid1])
  }
  
}





ANN.cv <- function(Y,X,nfold=10,rankmax=min(dim(Y),dim(X)),gamma=2,nlam=100,lambda=NULL,loglammin=0,norder=NULL,nest.tune=FALSE,fold.drop=0,tol=1e-4){

    n <- nrow(Y)
    p <- ncol(X)
    q <- ncol(Y)

    ndel <- round(n/nfold)
    if(is.null(norder)){
        norder <- sample(1:n,n)
    }
    
    if(is.null(rankmax)) rankmax <- min(p,q,n)
    ini <- RRR(Y,X,nrank=rankmax)
    C_ls <- ini$C_ls
    A <- ini$A
    Ad <- ini$Ad
    ###Note that eigen function may have numerical errors
    Ad <- Ad[Ad>tol]
    Adx <- (Ad)^((1+gamma)/2)


    cr_path <- matrix(ncol=nfold,nrow=nlam,NA)
    if(is.null(lambda)){
        lambda <- c(0,exp(seq(loglammin,log(max(Adx)),length=nlam-1)))
    }

    inifit <- list()            
    for(f in 1:nfold){
        ##determine cr sample
        if(f!=nfold){
            iddel <- norder[(1+ndel*(f-1)):(ndel*f)]
        }else{
            iddel <- norder[(1+ndel*(f-1)):n]
            ndel <- length(iddel)
        }
        nf <- n - ndel
        ##idkeep <- (1:n)[-iddel]
        ##print(paste("ndel=",ndel))
    
        Xf <- X[-iddel,]
        Xfdel <- X[iddel,]
        Yf <- Y[-iddel,]
        Yfdel <- Y[iddel,]

        rankmaxf <- min(rankmax,nf)
        inif <- RRR(Yf,Xf,nrank=rankmaxf)
        ##record the fitting
        inifit[[f]] <- inif
        
        C_lsf <- inif$C_ls
        Af <- inif$A
        Adf <- inif$Ad
        Adf <- Adf[Adf>tol]
        Adxf <- (Adf)^((1+gamma)/2)

        tempFit <- Xfdel%*%C_lsf 
        for(i in 1:nlam){
            dsth <- softTH(Adxf,lambda[i])/Adxf
            dsth[dsth < tol] <- 0
            rank <- sum(dsth!=0)
            ##df <- ifelse(dftype==1,sum(dsth),rank)
            ##SSd <- sum(((dsth-1)*sqrt(Adf))^2)
            if(rank!=0){
              tempC <- Af[,1:rank]%*%diag(dsth[1:rank],nrow=rank,ncol=rank)%*%t(Af[,1:rank])
              cr_path[i,f] <- sum((Yfdel-tempFit%*%tempC)^2)
            }else{             
              cr_path[i,f] <- sum(Yfdel^2)
            }
        }

    }

    index <- order(apply(cr_path,2,sum))[1:ifelse(nfold>5,nfold-fold.drop,nfold)]
    CRE <- apply(cr_path[,index],1,sum)
    id <- which.min(CRE)
   
  if(nest.tune==TRUE){
    ##############Refine estimation
    #################More precise search
    cr_path <- matrix(ncol=nfold,nrow=nlam,NA)
    lambda <- exp(seq(log(max(lambda[max(id-1,1)],tol)),log(lambda[min(id+1,nlam)]),length=nlam))
    ##note to reset ndel
    ndel <- round(n/nfold)
        
    for(f in 1:nfold){
        ##determine cr sample
        if(f!=nfold){
            iddel <- norder[(1+ndel*(f-1)):(ndel*f)]
        }else{
            iddel <- norder[(1+ndel*(f-1)):n]
            ndel <- length(iddel)
        }
        nf <- n - ndel
        ##idkeep <- (1:n)[-iddel]

        Xf <- X[-iddel,]
        Xfdel <- X[iddel,]
        Yf <- Y[-iddel,]
        Yfdel <- Y[iddel,]

        rankmaxf <- min(rankmax,nf)
        
        ##inif <- RRR(Yf,Xf,nrank=rankmaxf)
        inif <- inifit[[f]]
                
        C_lsf <- inif$C_ls
        Af <- inif$A
        Adf <- inif$Ad
        Adf <- Adf[Adf>tol]
        Adxf <- (Adf)^((1+gamma)/2)
        
        tempFit <- Xfdel%*%C_lsf 

        for(i in 1:nlam){
            dsth <- softTH(Adxf,lambda[i])/Adxf
            dsth[dsth < tol] <- 0            
            rank <- sum(dsth!=0)
            ##df <- ifelse(dftype==1,sum(dsth),rank)
            ##SSd <- sum(((dsth-1)*sqrt(Adf))^2)
            if(rank!=0){
              tempC <- Af[,1:rank]%*%diag(dsth[1:rank],nrow=rank,ncol=rank)%*%t(Af[,1:rank])
              cr_path[i,f] <- sum((Yfdel-tempFit%*%tempC)^2)
            }else{
                #####Note this is saying null solution is not allowed
                #####################################################              
                cr_path[i,f] <- sum(Yfdel^2)
            }
        }

    }
    
      
    index <- order(apply(cr_path,2,sum))[1:ifelse(nfold>5,nfold-fold.drop,nfold)]
    CRE <- apply(cr_path[,index],1,sum)
    id <- which.min(CRE)
  }
  
    dsth <- softTH(Adx,lambda[id])/Adx
    dsth[dsth < tol] <- 0
    rank <- sum(dsth!=0)
    if(rank!=0){
        C_sth <- C_ls%*%A[,1:rank]%*%diag(dsth[1:rank],nrow=rank,ncol=rank)%*%t(A[,1:rank])
    }else{
        C_sth <- matrix(nrow=p,ncol=q,0)
    }
    list(cr_path=cr_path, CRE=CRE, ID=id, norder=norder, lam=lambda[id], d=dsth[1:rank],C=C_sth,rank=rank)
}





####Proposed estimator, gamma=0, IDEAL TUNE
ANN.ideal <- function(Y,X,Ytest,Xtest,rankmax=min(dim(Y),dim(X)),gamma=2,nlam=100,lambda=NULL,loglammin=0,nest.tune=FALSE,tol=1e-4){

    n <- nrow(Y)
    p <- ncol(X)
    q <- ncol(Y)
    
    if(is.null(rankmax)) rankmax <- min(p,q,n)
    ini <- RRR(Y,X,nrank=rankmax)
    C_ls <- ini$C_ls
    A <- ini$A
    Ad <- ini$Ad
    Ad <- Ad[Ad>tol]

    ##SSEls <- sum((Y-X%*%C_ls)^2)
    #rankmax <- length(Ad)
    #A <- A[,1:length(Ad)]

    Adx <- (Ad)^((1+gamma)/2)
    
    if(is.null(lambda)){
        lambda <- c(0,exp(seq(loglammin,log(max(Adx)),length=nlam-1)))
    }    
        
    tempCC <- array(dim=c(q,q,nlam))
    cr_path <- vector()
    tempFit <- Xtest%*%C_ls
    for(i in 1:nlam){

        dsth <- softTH(Adx,lambda[i])/Adx
        rank <- sum(dsth!=0)
        
        if(rank!=0){
          tempCC[,,i] <- A[,1:rank]%*%diag(dsth[1:rank],nrow=rank,ncol=rank)%*%t(A[,1:rank])
          cr_path[i] <- sum((Ytest-tempFit%*%tempCC[,,i])^2)
        }else{
          tempCC[,,i] <- 0
          cr_path[i] <- sum(Ytest^2)
        }
    }
    id <- which.min(cr_path)
    
    ###Refine estimation
    ###More precise search
    
  if(nest.tune==TRUE){  
    tempCC <- array(dim=c(q,q,nlam))
    cr_path <- vector()
    lambda <- exp(seq(log(lambda[max(id-1,1)]),log(lambda[min(id+1,nlam)]),length=nlam))
    
    for(i in 1:nlam){

        dsth <- softTH(Adx,lambda[i])/Adx
        rank <- sum(dsth!=0)
        
        if(rank!=0){
          tempCC[,,i] <- A[,1:rank]%*%diag(dsth[1:rank],nrow=rank,ncol=rank)%*%t(A[,1:rank])
          cr_path[i] <- sum((Ytest-tempFit%*%tempCC[,,i])^2)
        }else{
          tempCC[,,i] <- 0
          cr_path[i] <- sum(Ytest^2)
        }
    }
    id <- which.min(cr_path)     
  }  
        
    dsth <- softTH(Adx,lambda[id])/Adx
    dsth[dsth < tol] <- 0
    C_sth <- C_ls%*%tempCC[,,id]
    rank_sth <- sum(dsth!=0)

    list(CRE=cr_path,C=C_sth,rank=rank_sth,lam=lambda[id])
}





######Robustified AST--------------------------------------###################
##lam1*|XC|_w* + lam2*|XC|_F^2
RANN.cv <- function(Y,X,nfold=10,rankmax=min(dim(Y),dim(X)),gamma=2,
                    nlam1=100,nlam2=100,lambda1=NULL,lambda2=seq(0,0.2,length=nlam2),
                    loglammin=0,norder=NULL,nest.tune=FALSE,fold.drop=0,tol=1e-4){

    n <- nrow(Y)
    p <- ncol(X)
    q <- ncol(Y)

    ndel <- round(n/nfold)
    if(is.null(norder)){
        norder <- sample(1:n,n)
    }
    cr_path <- array(dim=c(nlam1,nlam2,nfold),NA)

    if(is.null(rankmax)) rankmax <- min(p,q,n)
    ini <- RRR(Y,X,nrank=rankmax)
    C_ls <- ini$C_ls
    A <- ini$A
    Ad <- ini$Ad
    ###Note that eigen function may have numerical errors
    Ad <- Ad[Ad>tol]
    
    ##rankmax <- sum(Ad>tol)
    Adx <- (Ad)^((1+gamma)/2)

    ###lamstar=lam1/(1+2lam2)###
    if(is.null(lambda1)){
      lambda1 <- c(0,exp(seq(loglammin,log(max(Adx)),length=nlam1-1)))
    }  
    ##lamstar <- seq(0,max(Adx),length=nlam1) 

    inifit <- list()

    for(f in 1:nfold){
        ##determine cr sample
        if(f!=nfold){
            iddel <- norder[(1+ndel*(f-1)):(ndel*f)]
        }else{
            iddel <- norder[(1+ndel*(f-1)):n]
            ndel <- length(iddel)
        }
        nf <- n - ndel
        ##idkeep <- (1:n)[-iddel]

        Xf <- X[-iddel,]
        Xfdel <- X[iddel,]
        Yf <- Y[-iddel,]
        Yfdel <- Y[iddel,]

        rankmaxf <- min(rankmax,nf)
        inif <- RRR(Yf,Xf,nrank=rankmaxf)
        
        inifit[[f]] <- inif
        
        C_lsf <- inif$C_ls
        Af <- inif$A
        Adf <- inif$Ad
        Adf <- Adf[Adf>tol]
        Adxf <- (Adf)^((1+gamma)/2)

        tempFit <- Xfdel%*%C_lsf        
        for(i in 1:nlam1){
            dsth <- softTH(Adxf,lambda1[i])/Adxf
            rank <- sum(dsth!=0)
            ##df <- ifelse(dftype==1,sum(dsth),rank)
            ##SSd <- sum(((dsth-1)*sqrt(Adf))^2)
            if(rank!=0){
              tempC_sth <- Af[,1:rank]%*%diag(dsth[1:rank],nrow=rank,ncol=rank)%*%t(Af[,1:rank])
              fitdel <- tempFit%*%tempC_sth
              for(ll in 1:nlam2){                                         
                cr_path[i,ll,f] <- sum((Yfdel-fitdel/(1+2*lambda2[ll]))^2)
              }            
            }else{
                #####################################################              
                cr_path[i,,f] <- sum(Yfdel^2)
            }
        }

    }


    index <- order(apply(cr_path,3,sum))[1:ifelse(nfold>5,nfold-fold.drop,nfold)]
    crerr <- apply(cr_path[,,index],c(1,2),sum)
    minid <- which.min(crerr)
    minid2 <- ceiling(minid/nlam2)
    minid1 <- minid - nlam2*(minid2-1)

  if(nest.tune==TRUE){
    ###Update lambda
    lambda1 <- exp(seq(log(max(lambda1[max(minid1-1,1)],1e-4)),log(lambda1[min(minid1+1,nlam1)]),length=nlam1))
    lam2 <- seq(lambda2[max(minid2-1,1)],lambda2[min(minid2+1,nlam2)],length=nlam2)
    ndel <- round(n/nfold)
    
    for(f in 1:nfold){
        ##determine cr sample
        if(f!=nfold){
            iddel <- norder[(1+ndel*(f-1)):(ndel*f)]
        }else{
            iddel <- norder[(1+ndel*(f-1)):n]
            ndel <- length(iddel)
        }
        nf <- n - ndel
        ##idkeep <- (1:n)[-iddel]

        Xf <- X[-iddel,]
        Xfdel <- X[iddel,]
        Yf <- Y[-iddel,]
        Yfdel <- Y[iddel,]

        rankmaxf <- min(rankmax,nf)
        
        ##inif <- RRR(Yf,Xf,nrank=rankmaxf)
        inif <- inifit[[f]]
        
        C_lsf <- inif$C_ls
        Af <- inif$A
        Adf <- inif$Ad
        Adf <- Adf[Adf>tol]
        Adxf <- (Adf)^((1+gamma)/2)

        tempFit <- Xfdel%*%C_lsf        
        for(i in 1:nlam1){
            dsth <- softTH(Adxf,lambda1[i])/Adxf
            rank <- sum(dsth!=0)
            ##df <- ifelse(dftype==1,sum(dsth),rank)
            ##SSd <- sum(((dsth-1)*sqrt(Adf))^2)
            if(rank!=0){
              tempC_sth <- Af[,1:rank]%*%diag(dsth[1:rank],nrow=rank,ncol=rank)%*%t(Af[,1:rank])
              fitdel <- tempFit%*%tempC_sth
              for(ll in 1:nlam2){                                         
                cr_path[i,ll,f] <- sum((Yfdel-fitdel/(1+2*lambda2[ll]))^2)
              }            
            }else{
                #####Note this is saying null solution is not allowed
                #####################################################              
                cr_path[i,,f] <- sum(Yfdel^2)
            }
        }

    }

    index <- order(apply(cr_path,3,sum))[1:ifelse(nfold>5,nfold-fold.drop,nfold)]
    crerr <- apply(cr_path[,,index],c(1,2),sum)
    minid <- which.min(crerr)
    minid2 <- ceiling(minid/nlam2)
    minid1 <- minid - nlam2*(minid2-1)
  }
  
    dsth <- softTH(Adx,lambda1[minid1])/Adx
    dsth[dsth<tol] <- 0
    rank <- sum(dsth!=0)
    if(rank!=0){
        C_sth <- C_ls%*%A[,1:rank]%*%diag(dsth[1:rank],nrow=rank,ncol=rank)%*%t(A[,1:rank])/(1+2*lambda2[minid2])
    }else{
        C_sth <- matrix(nrow=p,ncol=q,0)
    }
    list(cr_path=cr_path, CRE=crerr,ID=c(minid1,minid2),norder=norder,lam=c(lambda1[minid1]*(1+2*lambda2[minid2]),lambda2[minid2]), C=C_sth,rank=rank)
}



######Ideally tuned Robustified AST--------------------------------------###################
##lam1*|XC|_w* + lam2*|XC|_F^2
RANN.ideal <- function(Y,X,Ytest,Xtest,rankmax=min(dim(Y),dim(X)),gamma=2,
                    nlam1=100,nlam2=100,lambda1=NULL,lambda2=seq(0,0.2,length=nlam2),
                    loglammin=0,nest.tune=FALSE,tol=1e-4){

    n <- nrow(Y)
    p <- ncol(X)
    q <- ncol(Y)

    cr_path <- matrix(nrow=nlam1,ncol=nlam2,NA)

    if(is.null(rankmax)) rankmax <- min(p,q,n)
    ini <- RRR(Y,X,nrank=rankmax)
    C_ls <- ini$C_ls
    A <- ini$A
    Ad <- ini$Ad
    ###Note that eigen function may have numerical errors
    Ad <- Ad[Ad>tol]
    
    ##rankmax <- sum(Ad>tol)
    Adx <- (Ad)^((1+gamma)/2)

    ###lambda1=lam1/(1+2lam2)###
    if(is.null(lambda1)){
      lambda1 <- c(0,exp(seq(loglammin,log(max(Adx)),length=nlam1-1)))
    }
    ##lamstar <- seq(0,max(Adx),length=nlam1) 

    tempFit <- Xtest%*%C_ls    
    for(i in 1:nlam1){
        dsth <- softTH(Adx,lambda1[i])/Adx
        rank <- sum(dsth!=0)

        if(rank!=0){
            tempC_sth <- A[,1:rank]%*%diag(dsth[1:rank],nrow=rank,ncol=rank)%*%t(A[,1:rank])
            fitnew <- tempFit%*%tempC_sth
            for(ll in 1:nlam2){                                         
                cr_path[i,ll] <- sum((Ytest-fitnew/(1+2*lambda2[ll]))^2)
            }            
        }else{
            cr_path[i,] <- sum(Ytest^2)
        }
    }

    minid <- which.min(cr_path)
    minid2 <- ceiling(minid/nlam2)
    minid1 <- minid - nlam2*(minid2-1)

  if(nest.tune==TRUE){
    ###Update lambda
    lambda1 <- exp(seq(log(lambda1[max(minid1-1,1)]),log(lambda2[min(minid1+1,nlam1)]),length=nlam1))
    lambda2 <- seq(lambda2[max(minid2-1,1)],lambda2[min(minid2+1,nlam2)],length=nlam2)
    for(i in 1:nlam1){
        dsth <- softTH(Adx,lambda1[i])/Adx
        rank <- sum(dsth!=0)

        if(rank!=0){
            tempC_sth <- A[,1:rank]%*%diag(dsth[1:rank],nrow=rank,ncol=rank)%*%t(A[,1:rank])
            fitnew <- tempFit%*%tempC_sth
            for(ll in 1:nlam2){                                         
                cr_path[i,ll] <- sum((Ytest-fitnew/(1+2*lambda2[ll]))^2)
            }            
        }else{
            cr_path[i,] <- sum(Ytest^2)
        }
    }
 
    minid <- which.min(cr_path)
    minid2 <- ceiling(minid/nlam2)
    minid1 <- minid - nlam2*(minid2-1)
  } 
 
    dsth <- softTH(Adx,lambda1[minid1])/Adx
    dsth[dsth<tol] <- 0
    rank <- sum(dsth!=0)
    if(rank!=0){
        C_sth <- C_ls%*%A[,1:rank]%*%diag(dsth[1:rank],nrow=rank,ncol=rank)%*%t(A[,1:rank])/(1+2*lambda2[minid2])
    }else{
        C_sth <- matrix(nrow=p,ncol=q,0)
    }
    list(CRE=cr_path,lam=c(lambda1[minid1]*(1+2*lambda2[minid2]),lambda2[minid2]),C=C_sth,rank=rank)
}











RRR.ic <- function(Y,X,rankmax=min(dim(Y),dim(X)),ic=1,ypy.svd=TRUE,c.svd=FALSE){

   q <- ncol(Y)
   n <- nrow(Y)
   p <- ncol(X)
   S_yx <- t(Y)%*%X
   S_xx <- t(X)%*%X

   ##S_xx_inv <- tryCatch(solve(S_xx+0.1*diag(p)),error=function(e)ginv(S_xx))
   S_xx_inv <- ginv(S_xx)
   if(sum(is.na(S_xx_inv))>0){
        S_xx_inv <- solve(S_xx+0.1*diag(p))
   }
   C_ls <- S_xx_inv%*%t(S_yx)
   
   if(ypy.svd){
     XC <- X%*%C_ls
     ##svdXC <- svd(XC,nu=nrank,nv=nrank)
     ##SVD may not converge
     svdXC <- tryCatch(svd(XC,nu=rankmax,nv=rankmax),error=function(e)2)
     if(tryCatch(svdXC==2,error=function(e)3)==3){
        A <- svdXC$v[,1:rankmax]
        Ad <- (svdXC$d[1:rankmax])^2
     }else{
       ypy.svd <- FALSE
     }
   }
   if(!ypy.svd){
     SS <- S_yx%*%C_ls
     SS <- (SS+t(SS))/2
     eigenSS <- eigen(SS,symmetric=TRUE)
     A <- as.matrix(eigenSS$vectors[,1:rankmax])
     Ad <- eigenSS$values[1:rankmax]   
   }
      
   ##Note that Ad is eigenvalue
   Ad <- round(Ad,4)
   rankmax <- sum(Ad!=0)
   if(length(Ad)<min(p,q)) Ad <- c(Ad,rep(0,min(p,q)-length(Ad)))
   
   xrank <- sum(svd(X)$d>1e-3)
   rmax <- min(xrank,q)
   
   ####Compute the exact degrees of freedom 
   df1 <- vector()   
   for(r in 1:rankmax){
  
      a <- vector()
      count = 1
      for(k in 1:r){      
         for(s in (r+1):rmax){
         ###for(s in (r+1):min(p,q)){         
            a[count] <- (Ad[s]+Ad[k])/(Ad[k]-Ad[s])
            count <- count +1
         }
      }
      term2 <- sum(a)
      ##h <- length(a)
      ##if(r==rankmax) term2 <- sum(a[-c(h-1,h)])
      if(r==rmax) term2 <- 0      
      ##if(r==rankmax & rankmax==min(p,q)) term2 <- 0      
      ##df1[r] <- max(p,q)*r + term2
      df1[r] <- max(xrank,q)*r + term2      
   }
   ##For full rank model and full rank fit  
   ##if(rankmax==min(p,q)) df1[rankmax] <- (p+q-rankmax)*rankmax
   
   ##compute naive df 
   df2 <- vector()
   for(r in 1:rankmax){
     df2[r] <- r*(xrank+q-r)
   }

   ####Information criterion
   sse <- vector()  
   ##IC using true df
   ##GCV, AIC, BIC, BICP
   IC1 <- matrix(ncol=4,nrow=rankmax)
   ##IC using naive df
   IC2 <- matrix(ncol=4,nrow=rankmax)
   
   ##Prediction error by test data
   ##predtest <- vector() 

   tempFit <- X%*%C_ls    
   tempC <- matrix(nrow=q,ncol=q,0)
   for(i in 1:rankmax){
      tempC <- tempC + A[,i]%*%t(A[,i])
      tempYhat <- tempFit%*%tempC
      sse[i] <- sum((Y-tempYhat)^2)
      ##predtest[i] <- sum((Ytest-tempYhat)^2)
      logsse <- log(sse[i])
      IC1[i,1] <- n*q*sse[i]/(n*q-df1[i])^2     
      IC1[i,2] <-  logsse + 2/q/n*df1[i] 
      IC1[i,3] <-  logsse + log(q*n)/q/n*df1[i]
      IC1[i,4] <- logsse + 2*log(p*q)/q/n*df1[i]  
      IC2[i,1] <- n*q*sse[i]/(n*q-df2[i])^2     
      IC2[i,2] <-  logsse + 2/q/n*df2[i] 
      IC2[i,3] <-  logsse + log(q*n)/q/n*df2[i]
      IC2[i,4] <- logsse + 2*log(p*q)/q/n*df2[i]             
   }

   rankest1 <- apply(IC1,2,which.min)  
   rankest2 <- apply(IC2,2,which.min)       
   
   ##nrank1 <- ifelse(exact==TRUE,rankest1[ic],rankest2[ic])
   nrank1 <- rankest1[ic]
   AA <- A[,1:nrank1]%*%t(A[,1:nrank1])
   C_rr1 <- C_ls%*%AA 

   nrank2 <- rankest2[ic]
   AA <- A[,1:nrank2]%*%t(A[,1:nrank2])
   C_rr2 <- C_ls%*%AA 
      
   if(c.svd){
     svd_C <- svd(C_rr1,nv=nrank1,nu=nrank1)
     U <- as.matrix(svd_C$u[,1:nrank1])
     V <- as.matrix(svd_C$v[,1:nrank1])
     D <- diag(svd_C$d[1:nrank1],nrow=nrank1)

     ##return ls estimator C_ls, reduced-rank estimator C_rr
     ##return SVD of C_rr
     list(A=A,Ad=Ad,C_ls=C_ls,C=C_rr1,U=U,V=V,D=D,C1=C_rr1,C2=C_rr2,rank=nrank1, 
         IC1=IC1,IC2=IC2, df1=df1, df2=df2, rankest1=rankest1,rankest2=rankest2,sse=sse)
   }else{
     list(A=A,Ad=Ad,C_ls=C_ls,C=C_rr1,C1=C_rr1,C2=C_rr2,rank=nrank1, 
         IC1=IC1, IC2=IC2, df1=df1, df2=df2, rankest1=rankest1,rankest2=rankest2,sse=sse)        
   }
   
}









ANN.ic <- function(Y,X,gamma=2,nlam=100,lambda=NULL,ic=1,
                    loglammin=0,tol=1e-4){

    n <- nrow(Y)
    p <- ncol(X)
    q <- ncol(Y)
    
    xrank <- sum(svd(X)$d>tol)
    rmax <- min(xrank,q)

    ##if(is.null(rankmax)) rankmax <- min(p,q,n)
    ini <- RRR(Y,X,nrank=rmax)
    C_ls <- ini$C_ls
    A <- ini$A
    Ad <- ini$Ad
    ###Note that eigen function may have numerical errors
    Ad <- Ad[Ad>tol]
        
    ##rankmax <- sum(Ad>tol)
    Adx <- (Ad)^((1+gamma)/2)

    if(is.null(lambda)){
        ##lambda <- c(exp(seq(log(max(Adx)),loglammin,length=nlam-1)),0)
        lambda <- exp(seq(log(max(Adx)),loglammin,length=nlam))        
    }


    ####Information criterion
    sse <- vector()  
    ##IC using true df
    ##GCV, AIC, BIC, BICP
    IC1 <- matrix(ncol=4,nrow=nlam)
    ##IC using naive df
    IC2 <- matrix(ncol=4,nrow=nlam)

    ##prediction error based on test data
    ##predtest <- vector()

    df1 <- vector()
    df2 <- vector()
    ##df3 <- vector()
    for(i in 2:nlam){
    
       f <- softTH(Adx,lambda[i])/Adx
       r <- sum(f!=0)
       f <- c(f,rep(0,min(p,q)-length(f)))
       Ad <- c(Ad,rep(0,min(p,q)-length(Ad)))
       
       ##term1 <- max(p,q)*sum(f)   
       term1 <- max(xrank,q)*sum(f)                
       
       a <- vector()
       count = 1
       for(k in 1:r){  
         #############################################################    
         ###for(s in (r+1):min(p,q)){
         for(s in (r+1):rmax){
            a[count] <- (Ad[k]*(f[k]-f[s])+Ad[s]*f[k])/(Ad[k]-Ad[s])
            count <- count +1
         }
       }
       term2 <- sum(a)
       h <- length(a)
       ##if(r==rankmax) term2 <- sum(a[-c(h-1,h)])
       if(r==rmax) term2 <- 0
       if(r==rmax & r==min(p,q)) term2 <- 0

       b <- vector()
       count = 1
       for(k in 1:r){      
         for(s in 1:r){
            if(s==k){
               b[count] <- 0
            }else{
               b[count] <- Ad[k]*(f[k]-f[s])/(Ad[k]-Ad[s])
            }
            count <- count +1
         }
       }
       term3 <- sum(b)    

       c <- vector()
       for(k in 1:r){
          c[k] <- lambda[i]*(gamma+1)*sqrt(Ad[k])^(-gamma-1)
          ##c[k] <- 2*lambda[i]*sqrt(Ad[k])^(-gamma)          
       }
       term4 <- sum(c)
       
#       for(k in 1:rmax){
#          c[k] <- 2*lambda[i]*(gamma+1)*sqrt(Ad[k])^(-gamma)
#       }
#       term4 <- sum(c)


#############################################################
       ###term4
       df1[i] <- term1 + term2 + term3 + term4
       df2[i] <- r*(xrank+q-r)
#############################################################

#       ###Candes's formular##
#       a <- vector()
#       for(k in 1:min(xrank,q)){
#         a[k] <- (Adx[k]>lambda[i])+abs(xrank-q)*f[k]
#       } 
#       term1 <- sum(a)
#       b <- vector()
#       count = 1
#       for(k in 1:min(xrank,q)){      
#         for(s in 1:min(xrank,q)){
#            if(s==k){
#               b[count] <- 0
#            }else{
#               b[count] <- Ad[k]*f[k]/(Ad[k]-Ad[s])
#            }
#            count <- count +1
#         }
#       }
#       term2 <- 2*sum(b)          
#       df3[i] <- term1 + term2
         
    }
    
    df1[1] <- 0
    df2[1] <- 0
       
    rankall <- vector() 
    tempFit <- X%*%C_ls 
    for(i in 1:nlam){
       dsth <- softTH(Adx,lambda[i])/Adx
       dsth[dsth < tol] <- 0
       rank <- sum(dsth!=0)
       rankall[i] <- rank
       ##df <- ifelse(dftype==1,sum(dsth),rank)
       ##SSd <- sum(((dsth-1)*sqrt(Adf))^2)
       if(rank!=0){
          tempC <- A[,1:rank]%*%diag(dsth[1:rank],nrow=rank,ncol=rank)%*%t(A[,1:rank])
          tempYhat <- tempFit%*%tempC
          sse[i] <- sum((Y-tempYhat)^2)
          ##predtest[i] <- sum((Ytest-tempYhat)^2)
       }else{
          sse[i] <- sum(Y^2)
          ##predtest[i] <- sum(Ytest^2)
       }
           
       logsse <- log(sse[i])
       IC1[i,1] <- n*q*sse[i]/(n*q-df1[i])^2     
       IC1[i,2] <-  logsse + 2/q/n*df1[i] 
       IC1[i,3] <-  logsse + log(q*n)/q/n*df1[i]
       IC1[i,4] <- logsse + 2*log(p*q)/q/n*df1[i]  
       IC2[i,1] <- n*q*sse[i]/(n*q-df2[i])^2     
       IC2[i,2] <-  logsse + 2/q/n*df2[i] 
       IC2[i,3] <-  logsse + log(q*n)/q/n*df2[i]
       IC2[i,4] <- logsse + 2*log(p*q)/q/n*df2[i]   
                  
    }

    lamest1 <- apply(IC1,2,which.min)  
    lamest2 <- apply(IC2,2,which.min) 
    rankest1 <- rankall[lamest1]
    rankest2 <- rankall[lamest2]
            
    ##id <- ifelse(exact==TRUE,lamest1[IC],lamest2[IC])
    id1 <- lamest1[ic]
    
    dsth <- softTH(Adx,lambda[id1])/Adx
    dsth[dsth < tol] <- 0
    rank1 <- sum(round(dsth,3)!=0)
    if(rank1!=0){
        C_sth1 <- C_ls%*%A[,1:rank1]%*%diag(dsth[1:rank1],nrow=rank1,ncol=rank1)%*%t(A[,1:rank1])
    }else{
        C_sth1 <- matrix(nrow=p,ncol=q,0)
    }


    id2 <- lamest2[ic]

    dsth <- softTH(Adx,lambda[id2])/Adx
    dsth[dsth < tol] <- 0
    rank2 <- sum(round(dsth,4)!=0)
    if(rank2!=0){
        C_sth2 <- C_ls%*%A[,1:rank2]%*%diag(dsth[1:rank2],nrow=rank2,ncol=rank2)%*%t(A[,1:rank2])
    }else{
        C_sth2 <- matrix(nrow=p,ncol=q,0)
    }



    list(lam1=lambda[id1],lam2=lambda[id2],d=dsth[1:rank],C=C_sth1,C1=C_sth1,C2=C_sth2,rank=rank1,rank1=rank1,rank2=rank2, 
         IC1=IC1, IC2=IC2, df1=df1, df2=df2, 
         lamest1=lamest1,lamest2=lamest2,
         rankall=rankall,rankest1=rankest1,rankest2=rankest2,sse=sse)        

}


#
#
###RRR.sim(n=100,p=25,q=25,snr=0.1,rstar=10,rx=25,rho.x=0.5,rho.e=0,SigmaX=CorrCS,SigmaE=CorrCS){
#
#
#n <- 100
#p <- 25
#q <- 25
#rho.x <- 0.5
#sim.data <- RRR.sim1(n=1000+n,p,q,snr=0.1,rho.x=0.9,rho.e=0)
#C <- sim.data$C
#Y <- sim.data$Y[1:n,]
#X <- sim.data$X[1:n,]
#Ytest <- sim.data$Y[101:1100,]
#Xtest <- sim.data$X[101:1100,]
#
#
#
#n <- 20
#p <- 100
#q <- 25
#rho.x <- 0.5
#sim.data <- RRR.sim(n=1000+n,p,q,snr=0.05,rstar=5,rx=10,rho.x=rho.x,rho.e=0)
#C <- sim.data$C
#Y <- sim.data$Y[1:20,]
#X <- sim.data$X[1:20,]
#Ytest <- sim.data$Y[21:1020,]
#Xtest <- sim.data$X[21:1020,]
#
#sum((X%*%RRR.ideal(Y,X,Ytest,Xtest)$C - X%*%C)^2)/q/n*100
#sum((X%*%RRRR.ideal(Y,X,Ytest,Xtest,nest.tune=FALSE)$C - X%*%C)^2)/q/n*100
#sum((X%*%RRRR.ideal(Y,X,Ytest,Xtest,nest.tune=TRUE)$C - X%*%C)^2)/q/n*100
#sum((X%*%ANN.ideal(Y,X,Ytest,Xtest,nest.tune=FALSE)$C - X%*%C)^2)/q/n*100
#sum((X%*%ANN.ideal(Y,X,Ytest,Xtest,nest.tune=TRUE)$C - X%*%C)^2)/q/n*100
#
#
#
#
#
#sum((Xtest%*%RRR(Y,X,10)$C - Ytest)^2)
#sum((Xtest%*%RRRR(Y,X,lambda=10,nrank=10)$C - Ytest)^2)
#sum((Xtest%*%RRR(Y,X,10,weight=TRUE,Gamma=solve(CorrCS(25,0.5)))$C - Ytest)^2)
#
#
#sum((Xtest%*%RRR.cv(Y,X)$C - Ytest)^2)
#sum((Xtest%*%RRRR.cv(Y,X,nest.tune=FALSE)$C - Ytest)^2)
#sum((Xtest%*%RRRR.cv(Y,X,nest.tune=TRUE)$C - Ytest)^2)
#sum((Xtest%*%ANN.cv(Y,X,nest.tune=FALSE)$C - Ytest)^2)
#sum((Xtest%*%ANN.cv(Y,X,nest.tune=TRUE)$C - Ytest)^2)
#sum((Xtest%*%RANN.cv(Y,X,nest.tune=FALSE)$C - Ytest)^2)
#sum((Xtest%*%RANN.cv(Y,X,nest.tune=TRUE)$C - Ytest)^2)
#
#sum((Xtest%*%RRR.ideal(Y,X,Ytest,Xtest)$C - Ytest)^2)
#sum((Xtest%*%RRRR.ideal(Y,X,Ytest,Xtest,nest.tune=FALSE)$C - Ytest)^2)
#sum((Xtest%*%RRRR.ideal(Y,X,Ytest,Xtest,nest.tune=TRUE)$C - Ytest)^2)
#sum((Xtest%*%ANN.ideal(Y,X,Ytest,Xtest,nest.tune=FALSE)$C - Ytest)^2)
#sum((Xtest%*%ANN.ideal(Y,X,Ytest,Xtest,nest.tune=TRUE)$C - Ytest)^2)
#
#
#n <- 100
#p <- 25
#q <- 25
#sim.data <- RRR.sim1(n=1000+n,p,q,snr=0.1,rho.x=0.9,rho.e=0)
#C <- sim.data$C
#Y <- sim.data$Y[1:n,]
#X <- sim.data$X[1:n,]
#Ytest <- sim.data$Y[101:1100,]
#Xtest <- sim.data$X[101:1100,]
#
#
#sum((X%*%RRR.ideal(Y,X,Ytest,Xtest)$C - X%*%C)^2)/q/n*100
#sum((X%*%RRRR.ideal(Y,X,Ytest,Xtest,nest.tune=FALSE)$C - X%*%C)^2)/q/n*100
#sum((X%*%RRRR.ideal(Y,X,Ytest,Xtest,nest.tune=TRUE)$C - X%*%C)^2)/q/n*100
#sum((X%*%ANN.ideal(Y,X,Ytest,Xtest,nest.tune=FALSE)$C - X%*%C)^2)/q/n*100
#sum((X%*%ANN.ideal(Y,X,Ytest,Xtest,nest.tune=TRUE)$C - X%*%C)^2)/q/n*100
#sum((X%*%RANN.ideal(Y,X,Ytest,Xtest,nest.tune=FALSE)$C - X%*%C)^2)/q/n*100
#sum((X%*%RANN.ideal(Y,X,Ytest,Xtest,nest.tune=TRUE)$C - X%*%C)^2)/q/n*100
#
#
#sum((Xtest%*%RRR.cv(Y,X)$C - Ytest)^2)
#sum((Xtest%*%RRR.ic(Y,X)$C - Ytest)^2)
#sum((Xtest%*%ANN.ic(Y,X)$C - Ytest)^2)
#
#
#
