\name{RRR-package}
\alias{RRR-package}
%\alias{RRR}
\docType{package}
\title{
    Reduced Rank Regression Methods
}
\description{
    This package implements several reduced rank methods for conducting multivariate linear regression.
}
\details{
  The following functions are available:
  \itemize{
    \item \code{RRR}: Perform reduced rank regression with a fixed rank.
  }

  The following functions are documented in the help pages:
  \itemize{
    \item \code{\link{RRR}}
  }


\tabular{ll}{
Package: \tab RRR\cr
Type: \tab Package\cr
Version: \tab 1.0\cr
Date: \tab 2013-04-12\cr
License: \tab GPL (>= 2)\cr
}


}
\author{
Kun Chen

Maintainer: Kun Chen <kuniowa@gamil.com>

}
\references{
  \itemize{
    \item Reinsel, G. C. and Velu, P. (1998) \emph{Multivariate Reduced-rank Regression: 
    Theory and Applications}. New York: Springer.    
    
    \item Bunea, F., She, Y. and Wegkamp, M. (2011) Optimal selection of reduced 
    rank estimators of high-dimensional matrices. \emph{The Annals of Statistics} 
    \bold{39}: 1282--1309.    

    \item Mukherjee, A., Zhu, J. (2011) Reduced rank Reduced rank ridge regression 
    and its kernel extensions. \emph{Statistical Analysis and Data Mining} \bold{4}: 612--622.

    \item Chen, K., Chan, K.-S. and Stenseth, N. C. (2012) Reduced rank stochastic regression
     with a sparse singular value decomposition. \emph{Journal of the Royal Statistical Society: Series B} 
     \bold{74}: 203--221.
    
    \item Chen, K., Dong, H. B. and Chan, K.-S. (2013) Reduced rank regression via 
    adaptive nuclear-norm penalization.
  }
}
\keyword{ package }
\seealso{

}
\examples{

##simulate data
n <- 100
p <- 25
q <- 25
sim.data <- RRR.sim1(n,p,q,rstar=10,snr=0.1,rho.x=0.5,rho.e=0.5)
C <- sim.data$C
Y <- sim.data$Y
X <- sim.data$X

##Unweighted RRR
sum((X\%*\%(RRR(Y,X,10)$C - C))^2)
##Weighted RRR. Use the true inverse covariance matrix as the weight matrix.
sum((X\%*\%(RRR(Y,X,10,weight=TRUE,Gamma=solve(CorrAR(q,0.5)))$C - C))^2)
    
}
