\name{RRR}
\alias{RRR}
\title{
Reduced Rank Regression Estimator
}
\description{
Compute reduced rank regression estimator for a given rank.
}
\usage{
RRR(Y, X, nrank = 1, weight = FALSE, Gamma = diag(ncol(Y)), 
    ypy.svd = TRUE, c.svd = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{Y}{
    An n times q matrix of response variables.
}
  \item{X}{
    An n times p matrix of predictors.
}
  \item{nrank}{
    Model rank.
}
  \item{weight}{
    If TRUE, then weighted criterion is performed.
}
  \item{Gamma}{
    An p times p weight matrix.
}
  \item{ypy.svd}{
    If TRUE, svd function is used.
}
  \item{c.svd}{
    If TRUE, output the svd of the coefficient matrix.
}
}
\details{
%%Compute reduced rank regression estimator for a given rank.
}
\value{
    \item{C}{The estimated coefficient matrix.}
    \item{C_ls}{The least squares estimator}
%% ...
}
\references{
  \itemize{
    \item Reinsel, G. C. and Velu, P. (1998) \emph{Multivariate Reduced-rank Regression: 
    Theory and Applications}. New York: Springer.     
    
    \item Chen, K., Dong, H. B. and Chan, K.-S. (2013) Reduced rank regression via 
    adaptive nuclear-norm penalization.
  }
}
\author{
Kun Chen
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{

##simulate data
n <- 100
p <- 25
q <- 25
sim.data <- RRR.sim1(n,p,q,rstar=10,snr=0.1,rho.x=0.5,rho.e=0.5)
C <- sim.data$C
Y <- sim.data$Y
X <- sim.data$X

##Unweighted RRR
sum((X\%*\%(RRR(Y,X,10)$C - C))^2)/q/n*100
##Weighted RRR. Use the true inverse covariance matrix as the weight matrix.
sum((X\%*\%(RRR(Y,X,10,weight=TRUE,Gamma=solve(CorrAR(q,0.5)))$C - C))^2)/q/n*100
    
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{regression}
%\keyword{reduced-rank}% __ONLY ONE__ keyword per line
